#!/usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const commander_1 = require("commander");
const viem_1 = require("viem");
const accounts_1 = require("viem/accounts");
const fs = __importStar(require("fs"));
const os = __importStar(require("os"));
const path = __importStar(require("path"));
const cli_table3_1 = __importDefault(require("cli-table3"));
const picocolors_1 = __importDefault(require("picocolors")); // Replace chalk import
const CONFIG_DIR = path.join(os.homedir(), ".shardeum-cli");
const CONFIG_PATH = path.join(CONFIG_DIR, "config");
// Utility functions
const truncateString = (str, maxLength = 122) => {
    return str.length > (maxLength - 2) ? `${str.substring(0, maxLength)}..` : str;
};
const formatOutput = (data) => {
    const table = new cli_table3_1.default({
        head: ["Property", "Value"],
    });
    const formatValue = (value) => {
        if (typeof value === "string")
            return truncateString(value);
        if (typeof value === "bigint")
            return value.toString();
        if (typeof value === "object" && value !== null) {
            // Convert any BigInt properties to strings in objects
            return JSON.stringify(value, (_, v) => typeof v === "bigint" ? v.toString() : v);
        }
        return JSON.stringify(value);
    };
    if (typeof data === "object") {
        Object.entries(data).forEach(([key, value]) => {
            table.push([key, formatValue(value)]);
        });
    }
    else {
        table.push(["Result", formatValue(data)]);
    }
    console.log(table.toString());
};
const handleError = (error) => {
    const table = new cli_table3_1.default({
        head: ["Error Detail", "Value"],
        colWidths: [30, 120],
    });
    table.push(["Message", error.message || "Unknown error"], ["Code", error.code || "N/A"], ["Reason", error.reason || "N/A"], ["Details", error.details || "N/A"]);
    console.error(table.toString());
};
// Configuration management
const loadConfig = () => {
    const defaultConfig = { rpcUrl: "https://atomium.shardeum.org", privateKey: "" };
    if (!fs.existsSync(CONFIG_PATH))
        return defaultConfig;
    try {
        return JSON.parse(fs.readFileSync(CONFIG_PATH, "utf-8"));
    }
    catch (_a) {
        return defaultConfig;
    }
};
const saveConfig = (config) => {
    if (!fs.existsSync(CONFIG_DIR)) {
        fs.mkdirSync(CONFIG_DIR, { recursive: true });
    }
    const currentConfig = loadConfig();
    const newConfig = Object.assign(Object.assign({}, currentConfig), config);
    fs.writeFileSync(CONFIG_PATH, JSON.stringify(newConfig, null, 2));
};
// CLI Program
const program = new commander_1.Command();
// Configuration commands
program
    .command("config:set-rpc")
    .description("Set RPC URL")
    .argument("<url>", "RPC URL")
    .action((url) => {
    saveConfig({ rpcUrl: url });
    formatOutput({ status: "RPC URL updated", url });
});
program
    .command("config:set-key")
    .description("Set private key")
    .argument("<key>", "Private key")
    .action((key) => {
    saveConfig({ privateKey: key });
    formatOutput({ status: "Private key updated" });
});
// Ethereum JSON-RPC commands
program
    .command("eth:blockNumber")
    .description("Get the latest block number")
    .action(() => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const config = loadConfig();
        const client = (0, viem_1.createPublicClient)({ transport: (0, viem_1.http)(config.rpcUrl) });
        const result = yield client.getBlockNumber();
        formatOutput({ blockNumber: result.toString() });
    }
    catch (error) {
        handleError(error);
    }
}));
program
    .command("eth:getBalance")
    .description("Get balance for an address")
    .argument("<address>", "Ethereum address")
    .action((address) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const config = loadConfig();
        const client = (0, viem_1.createPublicClient)({ transport: (0, viem_1.http)(config.rpcUrl) });
        const balance = yield client.getBalance({ address });
        formatOutput({
            address,
            balance: (0, viem_1.formatEther)(balance),
        });
    }
    catch (error) {
        handleError(error);
    }
}));
// eth_getStorageAt
program
    .command("eth:getStorageAt")
    .description("Get storage value at address and position")
    .argument("<address>", "Contract address")
    .argument("<position>", "Storage position")
    .action((address, position) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const config = loadConfig();
        const client = (0, viem_1.createPublicClient)({ transport: (0, viem_1.http)(config.rpcUrl) });
        const storage = yield client.getStorageAt({ address, slot: position });
        formatOutput({ address, position, storage });
    }
    catch (error) {
        handleError(error);
    }
}));
// eth_getTransactionCount
program
    .command("eth:getTransactionCount")
    .description("Get transaction count for address")
    .argument("<address>", "Account address")
    .action((address) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const config = loadConfig();
        const client = (0, viem_1.createPublicClient)({ transport: (0, viem_1.http)(config.rpcUrl) });
        const count = yield client.getTransactionCount({ address });
        formatOutput({ address, transactionCount: count.toString() });
    }
    catch (error) {
        handleError(error);
    }
}));
// eth_getCode
program
    .command("eth:getCode")
    .description("Get code at address")
    .argument("<address>", "Contract address")
    .action((address) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const config = loadConfig();
        const client = (0, viem_1.createPublicClient)({ transport: (0, viem_1.http)(config.rpcUrl) });
        const code = yield client.getBytecode({ address });
        formatOutput({ address, code });
    }
    catch (error) {
        handleError(error);
    }
}));
// eth_call
program
    .command("eth:call")
    .description("Execute contract call")
    .requiredOption("-t, --to <address>", "Contract address")
    .requiredOption("-d, --data <data>", "Call data")
    .option("-f, --from <address>", "From address")
    .action((options) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const config = loadConfig();
        const client = (0, viem_1.createPublicClient)({ transport: (0, viem_1.http)(config.rpcUrl) });
        const result = yield client.call(Object.assign({ to: options.to, data: options.data }, (options.from && { from: options.from })));
        formatOutput({ result: result.data });
    }
    catch (error) {
        handleError(error);
    }
}));
// eth_estimateGas
program
    .command("eth:estimateGas")
    .description("Estimate gas for transaction")
    .requiredOption("-t, --to <address>", "To address")
    .requiredOption("-d, --data <data>", "Transaction data")
    .option("-f, --from <address>", "From address")
    .option("-v, --value <value>", "Value in wei")
    .action((options) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const config = loadConfig();
        const client = (0, viem_1.createPublicClient)({ transport: (0, viem_1.http)(config.rpcUrl) });
        const gasEstimate = yield client.estimateGas(Object.assign(Object.assign({ to: options.to, data: options.data }, (options.from && { from: options.from })), (options.value && { value: BigInt(options.value) })));
        formatOutput({ gasEstimate: gasEstimate.toString() });
    }
    catch (error) {
        handleError(error);
    }
}));
// Block and Transaction related commands
program
    .command("eth:getBlockTransactionCountByHash")
    .description("Get block transaction count by hash")
    .argument("<hash>", "Block hash")
    .action((hash) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const config = loadConfig();
        const client = (0, viem_1.createPublicClient)({ transport: (0, viem_1.http)(config.rpcUrl) });
        const count = yield client.request({
            method: "eth_getBlockTransactionCountByHash",
            params: [hash],
        });
        formatOutput({ blockHash: hash, transactionCount: Number(count) });
    }
    catch (error) {
        handleError(error);
    }
}));
// Add remaining block and transaction commands
const blockCommands = [
    {
        name: "eth:getBlockByHash",
        description: "Get block by hash",
        method: "getBlock",
        params: (hash) => ({ blockHash: hash }),
    },
    {
        name: "eth:getBlockByNumber",
        description: "Get block by number",
        method: "getBlock",
        params: (number) => ({ blockNumber: BigInt(number) }),
    },
    {
        name: "eth:getTransactionByHash",
        description: "Get transaction by hash",
        method: "getTransaction",
        params: (hash) => ({ hash }),
    },
];
blockCommands.forEach(({ name, description, method, params }) => {
    program
        .command(name)
        .description(description)
        .argument("<value>", "Hash or block number")
        .action((value) => __awaiter(void 0, void 0, void 0, function* () {
        try {
            const config = loadConfig();
            const client = (0, viem_1.createPublicClient)({ transport: (0, viem_1.http)(config.rpcUrl) });
            const result = yield client[method](params(value));
            formatOutput(result);
        }
        catch (error) {
            handleError(error);
        }
    }));
});
// Network and utility commands
program
    .command("web3:clientVersion")
    .description("Get client version")
    .action(() => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const config = loadConfig();
        const client = (0, viem_1.createPublicClient)({ transport: (0, viem_1.http)(config.rpcUrl) });
        const version = yield client.request({ method: "web3_clientVersion" });
        formatOutput({ clientVersion: version });
    }
    catch (error) {
        handleError(error);
    }
}));
program
    .command("web3:sha3")
    .description("Calculate Keccak-256 hash")
    .argument("<data>", "Data to hash")
    .action((data) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const config = loadConfig();
        const client = (0, viem_1.createPublicClient)({ transport: (0, viem_1.http)(config.rpcUrl) });
        const hash = yield client.request({
            method: "web3_sha3",
            params: [data],
        });
        formatOutput({ input: data, hash });
    }
    catch (error) {
        handleError(error);
    }
}));
program
    .command("net:version")
    .description("Get network version")
    .action(() => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const config = loadConfig();
        const client = (0, viem_1.createPublicClient)({ transport: (0, viem_1.http)(config.rpcUrl) });
        const version = yield client.request({ method: "net_version" });
        formatOutput({ networkVersion: version });
    }
    catch (error) {
        handleError(error);
    }
}));
program
    .command("eth:protocolVersion")
    .description("Get Ethereum protocol version")
    .action(() => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const config = loadConfig();
        const client = (0, viem_1.createPublicClient)({ transport: (0, viem_1.http)(config.rpcUrl) });
        const version = yield client.request({ method: "eth_protocolVersion" });
        formatOutput({ protocolVersion: version });
    }
    catch (error) {
        handleError(error);
    }
}));
program
    .command("eth:sendTransaction")
    .description("Send a transaction")
    .requiredOption("-t, --to <address>", "To address")
    .option("-v, --value <value>", "Value in ether")
    .option("-d, --data <data>", "Transaction data")
    .option("-g, --gas <limit>", "Gas limit")
    .option("-p, --gasPrice <price>", "Gas price in gwei")
    .option("-n, --nonce <nonce>", "Nonce value")
    .action((options) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const config = loadConfig();
        if (!config.privateKey) {
            throw new Error("Private key not configured. Use config:set-key to set it.");
        }
        const account = (0, accounts_1.privateKeyToAccount)(`0x${config.privateKey.replace("0x", "")}`);
        const client = (0, viem_1.createWalletClient)({
            account,
            transport: (0, viem_1.http)(config.rpcUrl)
        });
        const transaction = Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({ to: options.to }, (options.value && { value: (0, viem_1.parseEther)(options.value) })), (options.data && { data: options.data })), (options.gas && { gas: BigInt(options.gas) })), (options.gasPrice && { gasPrice: (0, viem_1.parseGwei)(options.gasPrice) })), (options.nonce && { nonce: Number(options.nonce) }));
        const hash = yield client.sendTransaction(transaction);
        formatOutput(Object.assign(Object.assign({ transactionHash: hash, from: account.address }, transaction), (transaction.value && {
            value: `${options.value} ETH (${transaction.value.toString()} wei)`
        })));
    }
    catch (error) {
        handleError(error);
    }
}));
const createTestAccounts = () => {
    return Array(4).fill(0).map(() => {
        const privateKey = (0, accounts_1.generatePrivateKey)();
        return (0, accounts_1.privateKeyToAccount)(privateKey);
    });
};
const wait = (ms) => new Promise(resolve => setTimeout(resolve, ms));
const updateStats = (stats, lines) => {
    // Find the longest key length for padding
    const maxKeyLength = Math.max(...Object.keys(stats).map(k => k.length));
    process.stdout.write(`\x1b[${lines - 1}A\x1b[0J`);
    console.log("Test Progress:");
    Object.entries(stats).forEach(([key, value]) => {
        const paddedKey = key.padEnd(maxKeyLength);
        const successText = picocolors_1.default.green(value.success.toString());
        const errorText = picocolors_1.default.red(value.error.toString());
        console.log(`${paddedKey}: ${successText} / ${errorText}`);
    });
};
const initializeNonces = (client, accounts) => __awaiter(void 0, void 0, void 0, function* () {
    const nonces = {};
    yield Promise.all(accounts.map((account) => __awaiter(void 0, void 0, void 0, function* () {
        nonces[account.address] = yield client.getTransactionCount({
            address: account.address,
        });
    })));
    return nonces;
});
// Add the test command
program
    .command("test:network")
    .description("Run network test")
    .requiredOption("-t, --tps <number>", "Transactions per second")
    .requiredOption("-d, --duration <seconds>", "Test duration in seconds")
    .action((options) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const config = loadConfig();
        if (!config.privateKey) {
            throw new Error("Private key not configured");
        }
        const tps = parseInt(options.tps);
        const duration = parseInt(options.duration);
        const requestsPerTick = Math.ceil(tps / 4);
        const mainAccount = (0, accounts_1.privateKeyToAccount)(`0x${config.privateKey.replace("0x", "")}`);
        const testAccounts = createTestAccounts();
        const client = (0, viem_1.createPublicClient)({ transport: (0, viem_1.http)(config.rpcUrl) });
        const walletClient = (0, viem_1.createWalletClient)({
            account: mainAccount,
            transport: (0, viem_1.http)(config.rpcUrl),
        });
        // Initialize test accounts with some funds
        console.log("\n=== Initializing Test Accounts ===");
        for (const account of testAccounts) {
            try {
                const hash = yield walletClient.sendTransaction({
                    to: account.address,
                    value: (0, viem_1.parseEther)("0.01"),
                    chain: null,
                });
                console.log(`Funded ${account.address} (tx: ${hash})`);
                yield wait(1000); // Wait for transaction to be processed
            }
            catch (error) {
                console.error(`Failed to fund ${account.address}:`, error);
            }
        }
        // Rest of initialization code remains the same...
        console.log("\n=== Initializing Nonces ===");
        const nonceManager = new NonceManager();
        yield nonceManager.initialize(client, [...testAccounts, mainAccount]);
        console.log("\n=== Starting Network Test ===");
        // ...existing logging code...
        const stats = {
            sendTransaction: { success: 0, error: 0 },
            getTransactionCount: { success: 0, error: 0 },
            getBalance: { success: 0, error: 0 },
            blockNumber: { success: 0, error: 0 },
            getTransactionByHash: { success: 0, error: 0 }
        };
        // Initialize display
        Object.entries(stats).forEach(() => console.log());
        const statsLines = Object.keys(stats).length + 2;
        let lastStatsUpdate = Date.now();
        const startTime = Date.now();
        const endTime = startTime + (duration * 1000);
        // Modified request execution
        while (Date.now() < endTime) {
            const batchPromises = [];
            for (let i = 0; i < requestsPerTick; i++) {
                const targetAccount = testAccounts[Math.floor(Math.random() * testAccounts.length)];
                const senderAccount = testAccounts[Math.floor(Math.random() * testAccounts.length)];
                const operations = [];
                // Add a transaction
                operations.push({
                    type: 'sendTransaction',
                    operation: () => __awaiter(void 0, void 0, void 0, function* () {
                        const senderClient = (0, viem_1.createWalletClient)({
                            account: senderAccount,
                            transport: (0, viem_1.http)(config.rpcUrl),
                        });
                        const nonce = yield nonceManager.getNextNonce(senderAccount.address);
                        yield senderClient.sendTransaction({
                            to: targetAccount.address,
                            value: (0, viem_1.parseEther)("0.0001"),
                            chain: null,
                            nonce
                        });
                    })
                });
                // Add a random additional operation
                const op = Math.floor(Math.random() * 4);
                switch (op) {
                    case 0:
                        operations.push({
                            type: 'getTransactionCount',
                            operation: () => client.getTransactionCount({ address: targetAccount.address }).then(() => { })
                        });
                        break;
                    case 1:
                        operations.push({
                            type: 'getBalance',
                            operation: () => client.getBalance({ address: targetAccount.address }).then(() => { })
                        });
                        break;
                    case 2:
                        operations.push({
                            type: 'blockNumber',
                            operation: () => client.getBlockNumber().then(() => { })
                        });
                        break;
                    case 3:
                        operations.push({
                            type: 'getTransactionByHash',
                            operation: () => __awaiter(void 0, void 0, void 0, function* () {
                                const senderClient = (0, viem_1.createWalletClient)({
                                    account: senderAccount,
                                    transport: (0, viem_1.http)(config.rpcUrl),
                                });
                                const nonce = yield nonceManager.getNextNonce(senderAccount.address);
                                const hash = yield senderClient.sendTransaction({
                                    to: targetAccount.address,
                                    value: (0, viem_1.parseEther)("0.0001"),
                                    chain: null,
                                    nonce
                                });
                                yield client.getTransaction({ hash });
                            })
                        });
                        break;
                }
                // Execute operations
                for (const { type, operation } of operations) {
                    batchPromises.push(operation()
                        .then(() => { stats[type].success++; })
                        .catch(() => { stats[type].error++; }));
                }
            }
            // Wait for batch to complete
            yield Promise.allSettled(batchPromises);
            // Update stats display
            const now = Date.now();
            if (now - lastStatsUpdate >= 1000) {
                updateStats(stats, statsLines);
                lastStatsUpdate = now;
            }
            // Add small delay to prevent overwhelming the node
            yield wait(250);
        }
        // Rest of cleanup code remains the same...
        console.log("\n=== Test Complete ===");
        console.log("Returning balances to main account...");
        // Return balances
        for (const account of testAccounts) {
            try {
                const balance = yield client.getBalance({ address: account.address });
                if (balance > 0) {
                    const gasPrice = yield client.getGasPrice();
                    const gasLimit = 21000;
                    const gasCost = gasPrice * BigInt(gasLimit);
                    const transferAmount = balance - gasCost;
                    if (transferAmount > 0) {
                        const nonce = yield nonceManager.getNextNonce(account.address);
                        const hash = yield walletClient.sendTransaction({
                            to: mainAccount.address,
                            value: transferAmount,
                            account,
                            chain: null,
                            nonce
                        });
                        console.log(`Returned ${(0, viem_1.formatEther)(transferAmount)} ETH from ${account.address}`);
                    }
                }
            }
            catch (error) {
                if (error instanceof Error) {
                    console.error(`Failed to return balance from ${account.address}:`, error.message);
                }
                else {
                    console.error(`Failed to return balance from ${account.address}:`, error);
                }
            }
        }
        const finalBalance = yield client.getBalance({ address: mainAccount.address });
        console.log(`\nMain account final balance: ${(0, viem_1.formatEther)(finalBalance)} ETH`);
    }
    catch (error) {
        handleError(error);
    }
}));
program.parse(process.argv);
class NonceManager {
    constructor() {
        this.nonces = {};
        this.locks = {};
    }
    initialize(client, accounts) {
        return __awaiter(this, void 0, void 0, function* () {
            yield Promise.all(accounts.map((account) => __awaiter(this, void 0, void 0, function* () {
                this.nonces[account.address] = yield client.getTransactionCount({
                    address: account.address,
                });
                this.locks[account.address] = Promise.resolve();
            })));
        });
    }
    getNextNonce(address) {
        return __awaiter(this, void 0, void 0, function* () {
            // Wait for any pending operations on this address
            yield this.locks[address];
            // Create a new lock
            let resolveLock;
            const newLock = new Promise(resolve => { resolveLock = resolve; });
            this.locks[address] = newLock;
            try {
                const nonce = this.nonces[address]++;
                return nonce;
            }
            finally {
                resolveLock();
            }
        });
    }
}
